"""
Custom Base41 implementation as used for generating idlk file names.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

BASE41 = "0123456789abcdefghijklmnopqrstuvwxyz-_()$"
BASE41_DEC = [
    {char: idx * 41 * 41 * 41 * 41 * 41 for idx, char in enumerate(BASE41)},
    {char: idx * 41 * 41 * 41 * 41 for idx, char in enumerate(BASE41)},
    {char: idx * 41 * 41 * 41 for idx, char in enumerate(BASE41)},
    {char: idx * 41 * 41 for idx, char in enumerate(BASE41)},
    {char: idx * 41 for idx, char in enumerate(BASE41)},
    {char: idx for idx, char in enumerate(BASE41)}
]

class EncodeError(ValueError):
    """
    Raised whenever encoding fails.
    """
    pass

class DecodeError(ValueError):
    """
    Raised whenever decoding fails.
    """
    pass

def encode(value):
    """
    Encodes an integer value and returns a six-character string.
    """
    if value < 0 or value > 41**6-1:
        raise EncodeError('Value out of range')

    result = BASE41[value % 41]
    value //= 41
    result = BASE41[value % 41] + result
    value //= 41
    result = BASE41[value % 41] + result
    value //= 41
    result = BASE41[value % 41] + result
    value //= 41
    result = BASE41[value % 41] + result
    value //= 41
    result = BASE41[value] + result

    return result

def decode(data):
    """
    Decodes a six-character encoded value into an integer.
    """
    if len(data) != 6:
        raise DecodeError('Parameter must be a six-character string')
    try:
        return sum((BASE41_DEC[row][char] for row, char in enumerate(data, 0)))
    except (KeyError, IndexError):
        raise DecodeError('Data out of range')



"""
A lock filename generator for idlk files used by a well known DTP suite.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import os
import sys
import unicodedata
from idlk import base41

if sys.version_info[0] == 3:
    _get_byte = lambda c: c
else:
    _get_byte = ord

class IdlkError(ValueError):
    """
    Raised whenever filename generation fails.
    """
    pass

def hash_macroman(data):
    """
    Compute the hash for the given byte string.
    """
    result = 0
    for char in data:
        result = ((result << 8) + result) + _get_byte(char)

    return result % 0xFFFEECED

def idlk(filename):
    """
    Generate the lock file name for the given file.
    """

    # Normalize to NFC.
    filename = unicodedata.normalize('NFC', filename)

    # Convert to lowercase first.
    filename = filename.lower()

    # The original algorithm seems to prefer Mac Roman encoding as long as
    # there are no non-mappable characters in the file name.
    try:
        macroman_name = filename.encode("macroman")
    except UnicodeEncodeError:
        # Regrettably the encoding / hashing algorithm for unicode filenames is
        # not currently known. Please file a feature request/patch if you
        # discover a working implementation.
        raise IdlkError("File names with characters outside the Mac OS Roman "
                        "encoding are not supported")
    else:
        hashed = base41.encode(hash_macroman(macroman_name))
        base = os.path.splitext(macroman_name)[0]
        return "~{:s}~{:s}.idlk".format(base[0:18].decode("macroman"), hashed)



from setuptools import setup

setup(
    name='idlk',
    version='0.0.10',
    description='idlk lock filename generator',
    author='Lorenz Schori',
    author_email='lo@znerol.ch',
    url='https://github.com/znerol/py-idlk',
    packages=['idlk'],
    test_suite="idlk.test",
    zip_safe=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Plugins',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Multimedia :: Graphics'
    ]
)
